/* ------------------------------------------------------- */
/* основной файл устройства главные часы (RTC) mainrtc.cpp */
//******************************************************************************
//  Секция include: здесь подключается заголовочный файл к модулю
//******************************************************************************

#include "mainrtc.h" // заголовочный *.h файл текущего проекта

//------------------------------------------------------------------------

// транзитный счетчик контроля отказа линии внешнего
// секунд-генератора
volatile byte ErIrqRTC =0;
// передаточный флаг запроса обработчика очередной секунды
volatile byte SecF = 0;
// флаг запрета перезагрузки данных из RTC
volatile byte PGset = 0;
// флаг РВХ для функций возврата из настроек RTC
volatile byte TmpNumPozKur = 0;

// байт секунд внутреннего счета
volatile byte Clc_cntS = 0;
// байт минут внутреннего счета
volatile byte Clc_cntM = 0;
// байт часов внутреннего счета
volatile byte Clc_cntH = 0;
// временные байты обработки RTC
volatile byte tmp_rtc_sec = 0; // данные из адрес регистра секунд
volatile byte tmp_rtc_min = 0; // данные из адрес регистра минут
volatile byte tmp_rtc_hours = 0; // данные из адрес регистра часов
volatile byte tmp_rtc_day = 0; // данные из регистра дня недели
volatile byte tmp_rtc_date = 0; // данные из регистра даты (число)
volatile byte tmp_rtc_mons = 0; // данные из регистра месяца
volatile byte tmp_rtc_year = 0; // данные из регистра года
volatile char tmp_rtc_Tune = 0; // данные регистра коррекции

//------------------------------------------------------------------------
// свойства бодунов

//------------------------------------------------------------------------


//------------------------------------------------------------------------



//------------------------------------------------------------------------

// запуск секунд-генератора от RTC
void genSecInit()
{
 cli();
 attachInterrupt(digitalPinToInterrupt(L_OS), RtcSec, FALLING);
 sei();
}

// ежескундное прерывание от RTC
void RtcSec(){ SecF = 1; }

// функция счета времени
void clock()
{
 Clc_cntS++;
 if(Clc_cntS == 60)
  {
   Clc_cntS = 0; Clc_cntM++;
   if(Clc_cntM == 60)
    {
     Clc_cntM = 0; Clc_cntH++;
     if(Clc_cntH == 24){Clc_cntH = 0;}
    }
  }
}

// функция преобразования и вывода на индикатор
// данных главных часов
void vrClcDisp()
{ byte tmp;
 njx.v_ram[9] = fnt_bl;
 tmp = pgm_read_byte_near( BinDec + Clc_cntS );
 njx.v_ram[8] = njx.znak((tmp & B00001111));
 njx.v_ram[7] = njx.znak(((tmp>>4) & B00001111));
 njx.v_ram[6] = fnt_minus;
 tmp = pgm_read_byte_near( BinDec + Clc_cntM );
 njx.v_ram[5] = njx.znak((tmp & B00001111));
 njx.v_ram[4] = njx.znak(((tmp>>4) & B00001111));
 njx.v_ram[3] = fnt_minus;
 tmp = pgm_read_byte_near( BinDec + Clc_cntH );
 njx.v_ram[2] = njx.znak((tmp & B00001111));
 njx.v_ram[1] = njx.znak(((tmp>>4) & B00001111));
 njx.v_ram[0] = fnt_bl;
 njx.WRglass();
}

// запись одиночного байта в RTC по заданному адресу
// в аргументах идет адрес согласно карты RTC и байт данных
void Wrtc(byte adr, byte dat)
{byte erf=0;
 my_iic.mx_Start();
 erf=my_iic.mx_Txd(ds3231_W); // команда начать запись
 if(erf){erlinIIC();}
 erf=my_iic.mx_Txd(adr); // адрес данных
 if(erf){erlinIIC();}
 erf=my_iic.mx_Txd(dat); // данные
 if(erf){erlinIIC();}
 my_iic.mx_Stop();
}

// запись пары байт в RTC по заданному адресу
// в аргументах идет адрес согласно карты RTC и байт данных
void Wrtc2(byte adr, byte dat, byte dat1)
{byte erf=0;
 my_iic.mx_Start();
 erf=my_iic.mx_Txd(ds3231_W); // команда начать запись
 if(erf){erlinIIC();}
 erf=my_iic.mx_Txd(adr); // адрес данных
 if(erf){erlinIIC();}
 erf=my_iic.mx_Txd(dat); // данные
 if(erf){erlinIIC();}
 erf=my_iic.mx_Txd(dat1); // данные
 if(erf){erlinIIC();}
 my_iic.mx_Stop();
}

// чтение одиночного байта из RTC по заданному адресу
// возвращает принятый байт, аргументом идет адрес
// согласно карты RTC
byte Rrtc(byte adr)
{byte erf=0;
 my_iic.mx_Start();
 erf=my_iic.mx_Txd(ds3231_W); // команда начать запись
 if(erf){erlinIIC();}
 erf=my_iic.mx_Txd(adr); // адрес данных
 if(erf){erlinIIC();}
 my_iic.mx_reStart();
 erf=my_iic.mx_Txda(ds3231_R); // команда начать чтение
 if(erf){erlinIIC();}
 erf=my_iic.mx_Rxd(mx_noAck);
 my_iic.mx_Stop();
 return erf;
}

// функция блокировки по отсутствию тактовых
// секунд-импульсов от RTC
void ERROR_RTC()
{
 labelEror();
 // блокировка используемой силовой периферии
 // и источников команд(данных) ее управления

 njx.v_ram[5] = fnt_r; // загрузка прикладного фрагмента
 njx.v_ram[6] = fnt_c; // "rcLc"
 njx.v_ram[7] = fnt_L;
 njx.v_ram[8] = fnt_c;
 njx.WRglass();
 while (1)
 {
  // звуковой сигнал по желанию
 }
}

// функция ошибки обмена по I2C
void erlinIIC()
{
 labelEror();
 // блокировка используемой силовой периферии
 // и источников команд(данных) ее управления

 njx.v_ram[5] = fnt_r; // загрузка прикладного фрагмента
 njx.v_ram[6] = fnt_I; // "rI2C"
 njx.v_ram[7] = fnt_2;
 njx.v_ram[8] = fnt_C;
 njx.WRglass();
 while (1)
 {
  // звуковой сигнал по желанию
 }
}

/* первичная инициализация RTC DS3231 при
 первичной подаче питания и зажатой
 при включении питания(после аппаратного reset)
 кнопке RK */
extern void HardRTC_init()
{byte erf=0;
 my_iic.mx_Start();
 erf=my_iic.mx_Txd(ds3231_W); // команда начать запись
 if(erf){ erlinIIC(); }
 erf=my_iic.mx_Txd(0); // начальный адрес 0
 if(erf){ erlinIIC(); }
 erf=my_iic.mx_Txd(0); // 0 секунды главных часов
 if(erf){ erlinIIC(); }
 erf=my_iic.mx_Txd(0); // 1 минуты главных часов
 if(erf){ erlinIIC(); }
 erf=my_iic.mx_Txd(0); // 2 часы главных часов
 if(erf){ erlinIIC(); }
 erf=my_iic.mx_Txd(0x05); // 5 день недели (пятница) главных часов
 if(erf){ erlinIIC(); }
 erf=my_iic.mx_Txd(0x14); // 4 число месяца (14) главных часов
 if(erf){ erlinIIC(); }
 erf=my_iic.mx_Txd(0x04); // 5 месяц (4-апрель) главных часов
 if(erf){ erlinIIC(); }
 erf=my_iic.mx_Txd(0x23); // 6 год (23) главных часов
 if(erf){ erlinIIC(); }
 erf=my_iic.mx_Txd(0); // 7 секунды alarm1
 if(erf){ erlinIIC(); }
 erf=my_iic.mx_Txd(0); // 8 минуты alarm1
 if(erf){ erlinIIC(); }
 erf=my_iic.mx_Txd(0); // 9 часы alarm1
 if(erf){ erlinIIC(); }
 erf=my_iic.mx_Txd(0x01); // A число месяца alarm1
 if(erf){ erlinIIC(); }
 erf=my_iic.mx_Txd(0); // B минуты alarm2
 if(erf){ erlinIIC(); }
 erf=my_iic.mx_Txd(0); // C часы alarm2
 if(erf){ erlinIIC(); }
 erf=my_iic.mx_Txd(0x01); // D число месяца alarm2
 if(erf){ erlinIIC(); }
 erf=my_iic.mx_Txd(B01011000); // E выдача 1-секундного меандра
 if(erf){ erlinIIC(); }
 erf=my_iic.mx_Txd(0); // F
 if(erf){ erlinIIC(); }
 my_iic.mx_Stop();
 njx.BlankVram(fnt_bl);
 njx.v_ram[1]=fnt_r;
 njx.v_ram[2]=(1<<s_E)|(1<<s_F)|(1<<s_G);
 njx.v_ram[3]=fnt_C;
 njx.v_ram[5]=fnt_I;
 njx.v_ram[6]=fnt_n;
 njx.v_ram[7]=fnt_I;
 njx.v_ram[8]=(1<<s_E)|(1<<s_F)|(1<<s_G);
 njx.WRglass(); // вывод v_ram в индикатор
 tone(DTM, 500); delay(500);
 while(!(digitalRead(PinToV))){delay(100);}
 noTone(DTM);
}

// инициализация устройства "главные часы"
extern void DevClockInit()
{
 // очистка дисплея
 njx.BlankVram(fnt_bl);
 // разрешить вывод на индикацию главных часов
 DisClcInd = 0;
 // считать часы и календарь из RTC в блок копии хранения
 // только при первичной подаче питания
 // при переключении меджу "устройствами" не требуется
 if(!PGset)
 { 
  ReadClokBooc(); PGset = 1;
  // преобразовать секунды, минуты и часы в двоичный формат
  Clc_cntS = pgm_read_byte_near( DecBin + tmp_rtc_sec );
  Clc_cntM = pgm_read_byte_near( DecBin + tmp_rtc_min );
  Clc_cntH = pgm_read_byte_near( DecBin + tmp_rtc_hours );
  // и записать результат в соответствующие регистры главных часов
 }
 if(PGset){ WS_Lock = 1; }
 // провести вывод данных на индикацию
 vrClcDisp();
 // выполнить переприсвоение указателей комбинаций
 // к требуемому для основного статуса "устройства"
 // (индикация часов в ожидании команд клавиатуры)
 KeyPtrZ = SnZero; // указатель на функцию "все отпущены"
 KeyPtrL = dev1_init; // СЕКУНДОМЕР (указатель на функцию "<<")
 KeyPtrR = dev2_init; // ТАЙМЕР (указатель на функцию ">>")
 KeyPtrP = AdjRtcInit; // указатель на функцию "+"
 KeyPtrM = stub; // указатель на функцию "-"
 KeyPtrRK = stub; // указатель на функцию "RK"
 // загрузить указатель стека возврата
 RetSteckU = RetSteckH;
 RetSteckH = RetSteckL;
 RetSteckL = dev0_init; // он же DevClockInit
}

// функция "чтение часов и календаря"
void ReadClokBooc()
{
 byte error = 0;
 my_iic.mx_Start(); error = my_iic.mx_Txd(ds3231_W);
 if (error){ erlinIIC(); }
 error = my_iic.mx_Txd(adr_rtc_sec);
 if (error){ erlinIIC(); }
 my_iic.mx_reStart();
 error = my_iic.mx_Txda(ds3231_R);
 if (error){ erlinIIC(); }
 tmp_rtc_sec = my_iic.mx_Rxd(mx_Ack); // данные из адрес регистра секунд
 tmp_rtc_min = my_iic.mx_Rxd(mx_Ack); // данные из адрес регистра минут
 tmp_rtc_hours = my_iic.mx_Rxd(mx_Ack); // данные из адрес регистра часов
 tmp_rtc_day = my_iic.mx_Rxd(mx_Ack); // данные из регистра дня недели
 tmp_rtc_date = my_iic.mx_Rxd(mx_Ack); // данные из регистра даты (число)
 tmp_rtc_mons = my_iic.mx_Rxd(mx_Ack); // данные из регистра месяца
 tmp_rtc_year = my_iic.mx_Rxd(mx_noAck); // данные из регистра года
 my_iic.mx_Stop();
 tmp_rtc_Tune = Rrtc(adr_rtc_offset);
}

//------------------------------------------------------------------------

// просмотр/ввод параметров RTC и бодунов
// (методы редактирования параметров RTC и бодунов)

// инициализация окна выбора параметров
void AdjRtcInit()
{
 WS_Lock=1;
 DisClcInd = 1; // запрет индикации текущего времени
 njx.BlankVram(fnt_bl); // очистка видеопамяти
 // запись заставки окна выбора параметров редактирования
 njx.v_ram[0] = fnt_C;
 njx.v_ram[1] = fnt_0;
 njx.v_ram[2] = fnt_1;
 njx.v_ram[3] = fnt_2;
 njx.v_ram[4] = fnt_3;
 njx.v_ram[5] = fnt_4;
 njx.v_ram[6] = fnt_bl;
 njx.v_ram[7] = fnt_b;
 njx.v_ram[8] = fnt_1;
 njx.v_ram[9] = fnt_2;
 LdMas(EnKurF, LenPozDisp, 1);
 EnKurF[0] = 0; EnKurF[6] = 0; EnKurF[7] = 0;
 if(PGset){NumPozKur = 1;}
 else{NumPozKur = TmpNumPozKur;}
 njx.v_ram[NumPozKur] = njx.v_ram[NumPozKur] | fnt_coma;
 njx.WRglass();
 // записать новые значения в указатели кнопок
 KeyPtrZ = SnZero; // указатель на функцию "все отпущены"
 KeyPtrL = kurL; // указатель на функцию "<<"
 KeyPtrR = kurR; // указатель на функцию ">>"
 KeyPtrP = choiceP; // указатель на функцию "+"
 KeyPtrM = choiceM; // указатель на функцию "-"
 // записать в стек возврата указатель
 // на предшественника (DevClockInit)
 RetSteckU = RetSteckH; RetSteckH = RetSteckL;
 RetSteckL = dev0_init;
 if (PGset)
  {
  // считать данные из RTC только при PGset = 1
  ReadClokBooc(); PGset = 0;
  }
}


// инициализация окна редактирования секунд RTC
// (пуск по сигналу точного времени 00 секунд)
// "C0"
void STTim()
{
 njx.BlankVram(fnt_bl); // очистка видеопамяти
 // запись заставки окна выбора параметров редактирования
 njx.v_ram[0] = fnt_C;
 njx.v_ram[1] = fnt_0;
 njx.v_ram[3] = fnt_dnlin;
 njx.v_ram[4] = fnt_dnlin;
 njx.v_ram[5] = fnt_minus;
 njx.v_ram[6] = fnt_0;
 njx.v_ram[7] = fnt_0;
 njx.v_ram[9] = fnt_qest;
 LdMas(EnKurF, LenPozDisp, 0);
 EnKurF[9] = 1; NumPozKur = 9;
 njx.v_ram[NumPozKur] = njx.v_ram[NumPozKur] | fnt_coma;
 njx.WRglass();
}

// инициализация окна редактирования значений
// часов и минут RTC
// "C1"
void EditHM()
{
 njx.BlankVram(fnt_bl); // очистка видеопамяти
 // запись заставки окна выбора параметров редактирования
 njx.v_ram[0] = fnt_C;
 njx.v_ram[1] = fnt_1;
 njx.v_ram[5] = fnt_minus;
 njx.v_ram[9] = fnt_qest;
 njx.v_ram[7] = njx.znak((tmp_rtc_min & B00001111));
 njx.v_ram[6] = njx.znak(((tmp_rtc_min>>4) & B00001111));
 njx.v_ram[4] = njx.znak((tmp_rtc_hours & B00001111));
 njx.v_ram[3] = njx.znak(((tmp_rtc_hours>>4) & B00001111));
 LdMas(EnKurF, LenPozDisp, 0);
 EnKurF[9] = 1; EnKurF[7] = 1; EnKurF[6] = 1;
 EnKurF[4] = 1; EnKurF[3] = 1; NumPozKur = 9;
 njx.v_ram[NumPozKur] = njx.v_ram[NumPozKur] | fnt_coma;
 njx.WRglass();
}

// инициализация окна редактирования значений
// дня недели и числа RTC
// "C2"
void EditDD()
{
 njx.BlankVram(fnt_bl); // очистка видеопамяти
 // запись заставки окна выбора параметров редактирования
 njx.v_ram[0] = fnt_C;
 njx.v_ram[1] = fnt_2;
 njx.v_ram[5] = fnt_minus;
 njx.v_ram[9] = fnt_qest;
 njx.v_ram[7] = njx.znak((tmp_rtc_date & B00001111));
 njx.v_ram[6] = njx.znak(((tmp_rtc_date>>4) & B00001111));
 njx.v_ram[4] = njx.znak((tmp_rtc_day & B00001111));
 njx.v_ram[3] = njx.znak(((tmp_rtc_day>>4) & B00001111));
 LdMas(EnKurF, LenPozDisp, 0);
 EnKurF[9] = 1; EnKurF[7] = 1; EnKurF[6] = 1;
 EnKurF[4] = 1; EnKurF[3] = 1; NumPozKur = 9;
 njx.v_ram[NumPozKur] = njx.v_ram[NumPozKur] | fnt_coma;
 njx.WRglass();
}

// инициализация окна редактирования значений
// месяца и года RTC
// "C3"
void EditME()
{
 njx.BlankVram(fnt_bl); // очистка видеопамяти
 // запись заставки окна выбора параметров редактирования
 njx.v_ram[0] = fnt_C;
 njx.v_ram[1] = fnt_3;
 njx.v_ram[5] = fnt_minus;
 njx.v_ram[9] = fnt_qest;
 njx.v_ram[7] = njx.znak((tmp_rtc_year & B00001111));
 njx.v_ram[6] = njx.znak(((tmp_rtc_year>>4) & B00001111));
 njx.v_ram[4] = njx.znak((tmp_rtc_mons & B00001111));
 njx.v_ram[3] = njx.znak(((tmp_rtc_mons>>4) & B00001111));
 LdMas(EnKurF, LenPozDisp, 0);
 EnKurF[9] = 1; EnKurF[7] = 1; EnKurF[6] = 1;
 EnKurF[4] = 1; EnKurF[3] = 1; NumPozKur = 9;
 njx.v_ram[NumPozKur] = njx.v_ram[NumPozKur] | fnt_coma;
 njx.WRglass();
}

// инициализация окна редактирования значений
// корректора точности хода RTC
// "C4"
void EditTune()
{
 njx.BlankVram(fnt_bl); // очистка видеопамяти
 // запись заставки окна выбора параметров редактирования
 njx.v_ram[0] = fnt_C;
 njx.v_ram[1] = fnt_4;
 njx.v_ram[9] = fnt_qest;
 RDtmp_rtc_Tune();
 LdMas(EnKurF, LenPozDisp, 0);
 EnKurF[9] = 1; EnKurF[6] = 1; NumPozKur = 9;
 njx.v_ram[NumPozKur] = njx.v_ram[NumPozKur] | fnt_coma;
 njx.WRglass();
}

// инициализация окна редактирования значений
// параметров будильника 1
// "B1"
void EditB1()
{
 BlockBodus(); // тест-заглушка
}

// инициализация окна редактирования значений
// параметров будильника 2
// "B2"
void EditB2()
{
 BlockBodus(); // тест-заглушка
}


// кнопка "+" в режиме окна
// выбора параметров
void choiceP()
{
 // первичное исполнение (одиночное)
 if (!WS_Lock)
  {
   WS_Lock=1;
   // записать в стек возврата указатель
   // на предшественника (DevClockInit)
   RetSteckU = RetSteckH; RetSteckH = RetSteckL;
   RetSteckL = AdjRtcInit;
   KeyPtrP = EditP; // указатель на функцию "+"
   KeyPtrM = EditM; // указатель на функцию "-"
   // копируем позиционный номер параметра в РВХ
   TmpNumPozKur = NumPozKur;
   switch (NumPozKur) 
    {
    case 1:
      STTim();
      break;
    case 2:
      EditHM();
      break;
    case 3:
      EditDD();
      break;
    case 4:
      EditME();
      break;
    case 5:
      EditTune();
      break;
    case 8:
      EditB1();
      break;
    case 9:
      EditB2();
      break;
    default: 
      stub();
    break;
    }
  }
}

// кнопка "+" в режиме окна
// редактирования параметров
void EditP()
{
 // первичное исполнение (одиночное)
 if (!WS_Lock)
  {
   WS_Lock=1;
  if(NumPozKur == 9)
   { byte erinp = 0;
   // закрытие окна с записью новых параметров
   switch (TmpNumPozKur) 
    {
     case 1:
     // окна редактирования секунд RTC
     // (пуск по сигналу точного времени 00 секунд)
     // "C0"
        Wrtc(adr_rtc_sec, 0);
       break;
     case 2:
     // окна редактирования значений
     // часов и минут RTC
     // "C1"
       EditP_normC();
       if(njx.v_ram[6] >= 60){ erinp = 1; }
       if(njx.v_ram[3] >= 24){ erinp = 1; }
       if(!erinp)
        {
         tmp_rtc_min = njx.v_ram[7]; tmp_rtc_hours = njx.v_ram[4];
         Wrtc2(adr_rtc_min, njx.v_ram[7], njx.v_ram[4]);
        }
        else if(erinp){ErInp();}
       break;
     case 3:
      // окна редактирования значений
      // дня недели и числа RTC
      // "C2"
       EditP_normC();
       if(njx.v_ram[6] >= 32){ erinp = 1; }
       if(njx.v_ram[3] >= 8){ erinp = 1; }
       if(njx.v_ram[3] == 0){ erinp = 1; }
       if(!erinp)
        {
         tmp_rtc_date = njx.v_ram[7]; tmp_rtc_day = njx.v_ram[4];
         Wrtc2(adr_rtc_day, njx.v_ram[4], njx.v_ram[7]);
        }
        else if(erinp){ErInp();}
       break;
     case 4:
      // окна редактирования значений
      // месяца и года RTC
      // "C3"
       EditP_normC();
       if(njx.v_ram[3] >= 13){ erinp = 1; }
       if(!erinp)
        {
         tmp_rtc_year = njx.v_ram[7]; tmp_rtc_mons = njx.v_ram[4];
         Wrtc2(adr_rtc_mons, njx.v_ram[4], njx.v_ram[7]);
        }
        else if(erinp){ErInp();}
       break;
     case 5:
      // окна редактирования значений
      // корректора точности хода RTC
      // "C4"
        stub();
       break;
     case 8:
      // окна редактирования значений
      // параметров будильника 1
      // "B1"
        stub();
       break;
     case 9:
      // окна редактирования значений
      // параметров будильника 2
      // "B2"
        stub();
      break;
     default: 
       stub();
      break;
    }
    KeyPtr RetPtr = RetSteckL;
    RetSteckL = RetSteckH; RetSteckH = RetSteckU;
    RetPtr();
   }
  else
   {
    // обычный + с модулем 9 для RTC
    if(njx.v_ram[0] == fnt_C){ incKey(9); }
    // и селектор для бодунов
    if(njx.v_ram[0] == fnt_b)
    {
      
    }
    njx.WRglass();
   }
 
  }
}

// кнопка "-" в режиме окна
// выбора параметров
void choiceM()
{
 // первичное исполнение (одиночное)
 if (!WS_Lock)
  {
   WS_Lock=1;
   // возврат в предыдущую панель
   KeyPtr RetPtr = RetSteckL;
   RetSteckL = RetSteckH; RetSteckH = RetSteckU;
   RetPtr();
  }
}

// кнопка "-" в режиме окна
// редактирования параметров
void EditM()
{
 // первичное исполнение (одиночное)
 if (!WS_Lock)
  {
   WS_Lock=1;
   if(NumPozKur == 9)
    {
     // закрытие окна без записи новых параметров
     KeyPtr RetPtr = RetSteckL;
     RetSteckL = RetSteckH; RetSteckH = RetSteckU;
     RetPtr();
    }
   else
   {
    // обычный - с модулем 9 для RTC
    if(njx.v_ram[0] == fnt_C){ decKey(9); }
    // и селектор для бодунов
    if(njx.v_ram[0] == fnt_b)
    {
      
    }
    njx.WRglass();
   }
  }
}

// чтение байта tmp_rtc_Tune с
// нормированием для индикации
void RDtmp_rtc_Tune()
{
 byte tmp = tmp_rtc_Tune;
 if(bitRead(tmp, 7))
 {
  njx.v_ram[4] = fnt_minus;
  tmp = ( ~(tmp + 1));
 }
 if(tmp >= 100)
 { tmp = tmp - 100; njx.v_ram[5] = fnt_1; }
 tmp = pgm_read_byte_near( BinDec + tmp );
 njx.v_ram[7] = njx.znak((tmp & B00001111));
 njx.v_ram[6] = njx.znak(((tmp >> 4) & B00001111));
}

// фрагмент нормирования для проверки и ввода
// кнопа EditP для RTC часов/календаря
void EditP_normC()
{
 njx.v_ram[7] = njx.overZnak(njx.v_ram[7]);
 njx.v_ram[6] = njx.overZnak(njx.v_ram[6]);
 njx.v_ram[4] = njx.overZnak(njx.v_ram[4]);
 njx.v_ram[3] = njx.overZnak(njx.v_ram[3]);
 njx.v_ram[7] = njx.v_ram[7] | (njx.v_ram[6] << 4);
 njx.v_ram[4] = njx.v_ram[4] | (njx.v_ram[3] << 4);
 njx.v_ram[6] = pgm_read_byte_near( DecBin + njx.v_ram[7] );
 njx.v_ram[3] = pgm_read_byte_near( DecBin + njx.v_ram[4] );
}

// сигнал при ошибке диапазона ввода
void ErInp()
{
 njx.v_ram[3] = fnt_E;
 njx.v_ram[4] = fnt_r;
 njx.v_ram[5] = fnt_bl;
 njx.v_ram[6] = fnt_I;
 njx.v_ram[7] = fnt_n;
 njx.v_ram[8] = fnt_P;
 njx.v_ram[9] = fnt_bl;
 njx.WRglass();
 tone(DTM, 2000); delay(1000); noTone(DTM);
}

// тестовая заглушка бодунов
// (временно до их написания)
void BlockBodus()
{
 njx.BlankVram(fnt_bl); // очистка видеопамяти
 // запись заставки окна выбора параметров редактирования
 njx.v_ram[0] = fnt_b;
 njx.v_ram[1] = fnt_n;
 njx.v_ram[5] = fnt_minus;
 njx.v_ram[9] = fnt_qest;
 njx.v_ram[7] = fnt_trlin;
 njx.v_ram[6] = fnt_trlin;
 njx.v_ram[4] = fnt_trlin;
 njx.v_ram[3] = fnt_trlin;
 LdMas(EnKurF, LenPozDisp, 0);
 EnKurF[9] = 1; EnKurF[7] = 1; EnKurF[6] = 1;
 EnKurF[4] = 1; EnKurF[3] = 1; NumPozKur = 9;
 njx.v_ram[NumPozKur] = njx.v_ram[NumPozKur] | fnt_coma;
 njx.WRglass();
}

//------------------------------------------------------------------------



//------------------------------------------------------------------------
//-------------конец файла/end of file---------------------
